function make_atlas(directory)
	local geometries = {}

	local images = {}

	for _, filename in pairs(love.filesystem.enumerate(directory)) do
		if love.filesystem.isFile(directory..'/'..filename) then

			local name, ext = filename:match('(.+)%.(.+)$')
			ext = ext:lower()
			if ext == 'png' or ext == 'bmp' or ext == 'jpg' or ext == 'jpeg' or ext == 'gif' or ext == 'tga' then
				local imagedata = love.image.newImageData(directory..'/'..filename)
				local padded_imagedata = love.image.newImageData(imagedata:getWidth() + 2, imagedata:getHeight() + 2)

				padded_imagedata:paste(imagedata, 1, 1, 0, 0, imagedata:getWidth(), imagedata:getHeight())
				padded_imagedata:paste(imagedata, 0, 1, 0, 0, 1, imagedata:getHeight())
				padded_imagedata:paste(imagedata, padded_imagedata:getWidth()-1, 1, imagedata:getWidth()-1, 0, 1, imagedata:getHeight())
				padded_imagedata:paste(imagedata, 1, 0, 0, 0, imagedata:getWidth(), 1)
				padded_imagedata:paste(imagedata, 1, padded_imagedata:getHeight()-1, 0, imagedata:getHeight()-1, imagedata:getWidth(), 1)
				padded_imagedata:setPixel(0, 0, imagedata:getPixel(0, 0))
				padded_imagedata:setPixel(padded_imagedata:getWidth()-1, 0, imagedata:getPixel(imagedata:getWidth()-1, 0))
				padded_imagedata:setPixel(padded_imagedata:getWidth()-1, padded_imagedata:getHeight()-1, imagedata:getPixel(imagedata:getWidth()-1, imagedata:getHeight()-1))
				padded_imagedata:setPixel(0, padded_imagedata:getHeight()-1, imagedata:getPixel(0, imagedata:getHeight()-1))

				table.insert(images, {
					name = name,
					imagedata = padded_imagedata,
					width = padded_imagedata:getWidth(),
					height = padded_imagedata:getHeight(),
				})
			end
		end
	end

	table.sort(images, function(a, b) return a.height > b.height end)

	local total = 0
	local max_width = 0

	for i, image in ipairs(images) do
		total = total + image.width * image.height
		if image.width > max_width then
			max_width = image.width
		end
	end

	local lets_say_the_atlas_width_is_this

	if max_width > math.sqrt(total) then
		lets_say_the_atlas_width_is_this = max_width
	else
		lets_say_the_atlas_width_is_this = math.ceil(math.sqrt(total))
	end

	local this_height = 0
	local this_width = 0
	local width_of_atlas = 0
	local first_height = images[1] and images[1].height

	for i, image in ipairs(images) do
		if (this_width + image.width) > lets_say_the_atlas_width_is_this then
			this_width = 0
			this_height = this_height + first_height
			first_height = image.height
		end

		images[i].x = this_width
		images[i].y = this_height
		this_width = this_width + image.width

		if this_width > width_of_atlas then
			width_of_atlas = this_width
		end
	end

	local height_of_atlas = images[#images].y + first_height

	local atlas = love.image.newImageData(width_of_atlas, height_of_atlas)

	local geometries = {}

	for i, image in ipairs(images) do
		atlas:paste(image.imagedata, image.x, image.y, 0, 0, image.width, image.height)
		geometries[image.name] = love.graphics.newQuad(image.x+1, image.y+1, image.width-2, image.height-2, atlas:getWidth(), atlas:getHeight())
	end

	return love.graphics.newImage(atlas), geometries
end

function love.load()
	atlas, geometries = make_atlas("images")
end

function love.draw()
	love.graphics.drawq(atlas, geometries.grass, 10, 10)
end
